# Generated by Django 3.0.5 on 2021-06-25 16:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_drf_filepond', '0009_temporaryuploadchunked_offset_utime'),
    ]

    operations = [
        migrations.AlterField(
            model_name='temporaryuploadchunked',
            name='offset',
            field=models.BigIntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='temporaryuploadchunked',
            name='total_size',
            field=models.BigIntegerField(default=0),
        ),
    ]
