# Generated by Django 2.2.5 on 2020-04-17 15:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_drf_filepond', '0008_temporaryuploadchunked'),
    ]

    operations = [
        migrations.AddField(
            model_name='temporaryuploadchunked',
            name='offset',
            field=models.IntegerField(default=0),
        ),
        migrations.AlterField(
            model_name='temporaryuploadchunked',
            name='last_upload_time',
            field=models.DateTimeField(auto_now=True),
        ),
    ]
