# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-10-24 17:59
from __future__ import unicode_literals

import django.core.files.storage
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TemporaryUpload',
            fields=[
                ('file_id',
                 models.CharField(
                     max_length=22, primary_key=True, serialize=False,
                     validators=[
                         django.core.validators.MinLengthValidator(22)])),
                ('file',
                 models.FileField(
                     storage=django.core.files.storage.FileSystemStorage(
                         location=b'/tmp/filepond_uploads'), upload_to=b'')),
                ('upload_name', models.CharField(max_length=512)),
                ('uploaded', models.DateTimeField(auto_now_add=True)),
                ('upload_type',
                 models.CharField(
                     choices=[('F', 'Uploaded file data'),
                              ('U', 'Remote file URL')],
                     max_length=1)),
            ],
        ),
    ]
