# Generated by Django 2.2.5 on 2020-04-16 15:21

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_drf_filepond', '0007_altered_stored_upload_file'),
    ]

    operations = [
        migrations.CreateModel(
            name='TemporaryUploadChunked',
            fields=[
                ('upload_id',
                 models.CharField(
                     max_length=22, primary_key=True, serialize=False,
                     validators=[
                         django.core.validators.MinLengthValidator(22)])),
                ('file_id',
                 models.CharField(
                     max_length=22,
                     validators=[
                         django.core.validators.MinLengthValidator(22)])),
                ('upload_dir',
                 models.CharField(
                     default=models.CharField(
                         max_length=22, primary_key=True, serialize=False,
                         validators=[
                             django.core.validators.MinLengthValidator(22)]),
                     max_length=512)),
                ('last_chunk', models.IntegerField(default=0)),
                ('total_size', models.IntegerField(default=0)),
                ('upload_name', models.CharField(default='', max_length=512)),
                ('upload_complete', models.BooleanField(default=False)),
                ('last_upload_time', models.DateTimeField(auto_now_add=True)),
                ('uploaded_by',
                 models.ForeignKey(
                     blank=True, null=True,
                     on_delete=django.db.models.deletion.CASCADE,
                     to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
