# Generated by Django 3.0.3 on 2020-02-21 18:59

from django.db import migrations, models
import django_drf_filepond.models


class Migration(migrations.Migration):

    dependencies = [
        ('django_drf_filepond', '0006_permupload_mods'),
    ]

    operations = [
        migrations.AlterField(
            model_name='storedupload',
            name='file',
            field=models.FileField(
                max_length=2048,
                storage=django_drf_filepond.models.DrfFilePondStoredStorage(),
                upload_to=''),
        ),
    ]
