# Generated by Django 2.1.3 on 2018-12-04 14:50
import django.core.validators
from django.db import migrations, models
import shortuuid


class Migration(migrations.Migration):

    dependencies = [
        ('django_drf_filepond', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='temporaryupload',
            old_name='file_id',
            new_name='upload_id',
        ),
        migrations.AddField(
            model_name='temporaryupload',
            name='file_id',
            field=models.CharField(
                default=shortuuid.uuid(), max_length=22, serialize=False,
                validators=[django.core.validators.MinLengthValidator(22)]),
            preserve_default=False,
        ),
    ]
