# Generated by Django 2.1.3 on 2019-02-13 13:00
import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_drf_filepond', '0002_add_upload_dir'),
    ]

    operations = [
        migrations.CreateModel(
            name='StoredUpload',
            fields=[
                ('upload_id',
                 models.CharField(
                     max_length=22, primary_key=True, serialize=False,
                     validators=[
                         django.core.validators.MinLengthValidator(22)])),
                ('file_path', models.CharField(max_length=2048)),
                ('uploaded', models.DateTimeField()),
                ('stored', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]
